# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.lite.constants namespace.
"""

from __future__ import print_function as _print_function

import sys as _sys

from tensorflow.lite.python.lite_constants import FLOAT
from tensorflow.lite.python.lite_constants import FLOAT16
from tensorflow.lite.python.lite_constants import GRAPHVIZ_DOT
from tensorflow.lite.python.lite_constants import INT32
from tensorflow.lite.python.lite_constants import INT64
from tensorflow.lite.python.lite_constants import INT8
from tensorflow.lite.python.lite_constants import QUANTIZED_UINT8
from tensorflow.lite.python.lite_constants import STRING
from tensorflow.lite.python.lite_constants import TFLITE

del _print_function
